using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using System.Collections.Specialized;
using System.Reflection;

using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Grouping;

using Jefferies.IPM.UI;
using Jefferies.IPM.UI.Module.NewIssue.Model;
//using Jefferies.Core.Data;
using Jefferies.Common.Data.Sql.DataAccess;
using Jefferies.Common.Data.Collections.GridHelper;

namespace Jefferies.IPM.UI.Module.NewIssue
{
	public partial class NewIssueForm : Form
	{
		public NewIssueForm(string bookEntity, string desk)
		{
			InitializeComponent();
		}

		private static class Columns
		{
			public static string Moniker = "Mnkr";
			public static string MonikerName = "MnkrName";
			public static string BookEntity = "BookEntity";
			public static string Comments	= "Comments";
			public static string OrderPercentage = "OrderPercentage";
			public static string OrderQuantity = "OrderQuantity";
			public static string AllocQuantity = "AllocatedQuantity";
			public static string StockOrderQuantity = "StockOrderQuantity";
			public static string StockAllocQuantity = "StockAllocatedQuantity";
			public static string AE = "AE";
			public static string Capacity = "Capacity";
			public static string AllocPercentage = "AllocatedPercentage";
			public static string RequestedPercentage = "RequestedPercentage";
			public static string Cheap = "Cheap";
			public static string Mids = "Mids";
			public static string Rich = "Rich";
            public static string RequestedHedge = "RequestedHedge";
            public static string ReqStockAllocQty = "ReqStockAllocQty";
            public static string FinalStockAllocQty = "FinalStockAllocQty";
        }

		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);
			this.SetLookAndFeelSpreadsheet();
			this.SetUpSpreadsheetGrid();
			this.ApplyMetaDataSpreadsheet();
			this.LoadNewIssueOrderList(1);

            _gridSpreadsheet.TableControl.HScrollBehavior = GridScrollbarMode.Enabled;
		}

		protected virtual void SetLookAndFeelSpreadsheet()
		{
			_gridSpreadsheet.Appearance.GroupCaptionCell.BackColor = SystemColors.Window;
			_gridSpreadsheet.Appearance.GroupCaptionCell.Borders.Top = GridBorder.Empty;

			_gridSpreadsheet.Appearance.AnyCell.Borders.Left = GridBorder.Empty;
			_gridSpreadsheet.Appearance.AnyCell.Borders.Top = GridBorder.Empty;
			_gridSpreadsheet.Appearance.AnyCell.Borders.Right = new GridBorder(GridBorderStyle.Solid, Color.Gainsboro, GridBorderWeight.Thin);
			_gridSpreadsheet.Appearance.AnyCell.Borders.Bottom = new GridBorder(GridBorderStyle.Solid, Color.Gainsboro, GridBorderWeight.Thin);

			_gridSpreadsheet.Appearance.AnyHeaderCell.Borders.All = new GridBorder(GridBorderStyle.None);

			_gridSpreadsheet.TableOptions.RecordRowHeight = 17;
			_gridSpreadsheet.TableOptions.RecordPreviewRowHeight = 17;
			_gridSpreadsheet.TableOptions.CaptionRowHeight = 17;
			_gridSpreadsheet.TableOptions.ColumnHeaderRowHeight = 20;

			_gridSpreadsheet.TopLevelGroupOptions.ShowCaption = false;

			_gridSpreadsheet.ContextMenuStrip = new ContextMenuStrip();

			_gridSpreadsheet.ShowGroupDropArea = false;
			_gridSpreadsheet.TopLevelGroupOptions.ShowFilterBar = false;

			_gridSpreadsheet.TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;
			_gridSpreadsheet.TopLevelGroupOptions.ShowAddNewRecordAfterDetails = true;

			_gridSpreadsheet.Table.RecordValueChanging += new RecordValueChangingEventHandler(Table_RecordValueChanging);
			_gridSpreadsheet.Table.CurrentRecordContextChange += new CurrentRecordContextChangeEventHandler(Table_CurrentRecordContextChange);
		}

		private void Table_CurrentRecordContextChange(object sender, CurrentRecordContextChangeEventArgs e)
		{
			if (e.Action == CurrentRecordAction.EndEditCalled && e.Record.Kind == DisplayElementKind.AddNewRecord)
			{
				//NewIssueOrderItem item	= (NewIssueOrderItem) e.Record.GetData();
				//((IEditableObject)item).EndEdit();
				//e.Cancel = true;
				//e.Table.CurrentRecord.
				//e.Table.CurrentRecord.EndEdit();
			}
			//throw new Exception("The method or operation is not implemented.");
			//e.Action == CurrentRecordAction.CancelEditCalled;
		}

		private void SetUpSpreadsheetGrid()
		{
			_gridSpreadsheet.ContextMenuStrip.Items.Add("Delete Row", null, new EventHandler(OnDeleteRow));
		}

		private void OnDeleteRow(object sender, EventArgs args)
		{
			if (true)
			{
				DialogResult result	= MessageBox.Show("Do you want to delete the selected row?", "Delete Row", MessageBoxButtons.OKCancel);

				if (result == DialogResult.OK)
				{
					if (_gridSpreadsheet.Table.CurrentElement != null)
					{
						Record record = _gridSpreadsheet.Table.CurrentElement.GetRecord();
						if (record is GridRecord)
						{
							record.Delete();
							_gridSpreadsheet.Table.Reload();
							_gridSpreadsheet.Invalidate();
							_gridSpreadsheet.Refresh();
						}
					}
				}
			}
		}

		private void ApplyMetaDataSpreadsheet()
		{
			_gridSpreadsheet.TableDescriptor.Columns.Clear();
			RowItemDescriptor itemDescriptor	= new RowItemDescriptor(typeof(NewIssueOrderItem));

			foreach (RowItemFormatAttribute attr in itemDescriptor.GetFormatAttributes())
			{
				PropertyInfo property = itemDescriptor.GetPropertyInfoForAttribute(attr);

				GridColumnDescriptor descriptor = new GridColumnDescriptor();
				descriptor.MappingName = property.Name;
				descriptor.Name = attr.Name;
				descriptor.AllowFilter = attr.AllowFilter;
				descriptor.Width = attr.Width;
				descriptor.ReadOnly				= attr.ReadOnly;

				if (attr.Format != null)
				{
					descriptor.Appearance.AnyRecordFieldCell.Format = attr.Format;
				}

				GridHorizontalAlignment hAlign = GridHorizontalAlignment.Center;
				if (attr.HAlign == RowItemHAlign.Right)
				{
					hAlign = GridHorizontalAlignment.Right;
				}
				else if (attr.HAlign == RowItemHAlign.Left)
				{
					hAlign = GridHorizontalAlignment.Left;
				}
				descriptor.Appearance.AnyRecordFieldCell.HorizontalAlignment = hAlign;
				descriptor.Appearance.AnyNestedTableCell.HorizontalAlignment = hAlign;

				_gridSpreadsheet.TableDescriptor.Columns.Add(descriptor);
			}

			for (int i = 0; i < _gridSpreadsheet.TableDescriptor.Columns.Count; i++)
			{
				_gridSpreadsheet.TableModel[0, i].Text = _gridSpreadsheet.TableDescriptor.Columns[i].Name;
			}
			_gridSpreadsheet.TableModel.Cols.FreezeRange(0, 1);
		}



		private void LoadNewIssueOrderList(int newIssueId)
		{
			NewIssueOrderItemList list = new NewIssueOrderItemList();
			NewIssueOrder newIssueOrder;

			for (int i = 0; i < 20; i++)
			{
				newIssueOrder = new NewIssueOrder();
				newIssueOrder.AE	= "AE"+i.ToString();
				newIssueOrder.Capacity	= "A";
				newIssueOrder.Mnkr	= "Mnkr"+i.ToString();

				list.Add(new NewIssueOrderItem(newIssueOrder));
				list[list.Count - 1].IsNew = false;
			}

			_gridSpreadsheet.DataSource		= list;
		}

		private void Table_RecordValueChanging(object sender, RecordValueChangingEventArgs e)
		{
			bool isValid = true;
			//validations
			this._gridSpreadsheet.TableControl.CurrentCell.ConfirmChanges();
			if (e.FieldDescriptor.Name == Columns.Moniker)
			{
				if (e.NewValue.ToString().Length > 10 || e.NewValue.ToString() == String.Empty)
				{
					isValid = false;
				}
				else
				{
                    e.NewValue = e.NewValue.ToString().ToUpper();
                    e.Record.SetValue(Columns.MonikerName, "Test "+e.NewValue.ToString());
					//e.Record.SetValue(Columns.AE, moniker.Sales.ToUpper());
					e.Record.SetValue(Columns.Capacity, "A");
				}
			}

			if (e.FieldDescriptor.Name == Columns.BookEntity)
			{
				if (e.NewValue.ToString().Length > 3)
				{
					isValid = false;
				}
			}

			if (e.FieldDescriptor.Name == Columns.Comments)
			{
				if (e.NewValue.ToString().Length > 100)
				{
					isValid = false;
				}
			}

			if (e.FieldDescriptor.Name == Columns.AE)
			{
				if (e.NewValue.ToString().Length > 3)
				{
					isValid = false;
				}
			}

			if (e.FieldDescriptor.Name == Columns.Capacity)
			{
				if (e.NewValue.ToString().Length > 1)
				{
					isValid = false;
				}
				e.NewValue = e.NewValue.ToString().ToUpper();
				if (e.NewValue.ToString() != "A" && 
                    e.NewValue.ToString() != "O")
				{
					isValid = false;
				}
			}

			if (!isValid)
			{
				MessageBox.Show("Entered value is not consistent with the underlying data-type. Please try again.");
				e.Cancel = true;
			}
		}

		private void panel2_Paint(object sender, PaintEventArgs e)
		{

		}

		private void splitNewIssue_Panel1_Paint(object sender, PaintEventArgs e)
		{

		}

		
	}
}